package com.simuwang.manage.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.simuwang.base.common.conts.ExcelConst;
import com.simuwang.base.common.enums.DistributeType;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.daq.InvestmentManagerMapper;
import com.simuwang.base.mapper.daq.ProductContractMapper;
import com.simuwang.base.mapper.daq.ProductDerivativeMapper;
import com.simuwang.base.mapper.daq.ProductInformationMapper;
import com.simuwang.base.pojo.dos.*;
import com.simuwang.base.pojo.dto.*;
import com.simuwang.base.pojo.dto.query.ProductPageQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.manage.service.ProductService;
import com.simuwang.shiro.utils.UserUtils;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ProductServiceImpl implements ProductService {
    @Autowired
    private ProductInformationMapper productInformationMapper;

    @Autowired
    private ProductContractMapper productContractMapper;

    @Autowired
    private ProductDerivativeMapper productDerivativeMapper;

    @Autowired
    private InvestmentManagerMapper investmentManagerMapper;

    private static final Logger logger = LoggerFactory.getLogger(ProductServiceImpl.class);
    @Value("${email.file.path}")
    private String path;
    @Override
    public MybatisPage<ProductInformationVO> searchProductList(ProductPageQuery productPageQuery) {
        List<ProductInformationDO> productInformationDOList = productInformationMapper.searchProductList(productPageQuery);
        List<ProductInformationVO> productInformationVOList =  productInformationDOList.stream().map(ProductInformationDO::toVO).collect(Collectors.toList());
        long total = productInformationMapper.countProductList(productPageQuery);
        return MybatisPage.of(total,productInformationVOList);
    }

    @Override
    public ResultVo uploadProduct(MultipartFile excelFile) {
        ResultVo vo = new ResultVo(ResultCode.SUCCESS);
        File file = null;
        try{
            InputStream inputStream = excelFile.getInputStream();
            file = new File(path+"/upload/"+ System.currentTimeMillis()+"/"+excelFile.getOriginalFilename());
            FileUtils.copyInputStreamToFile(inputStream,file);
            ProductData productData = parseProductFile(file);
            vo.setData(parseResult(productData));
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            vo.setMsg("文件解析异常");
            vo.setData(false);
            return vo;
        }

        return vo;
    }

    @Override
    public ResultVo productDetail(IdVO idVO) {
        ProductDataVO productDataVO = new ProductDataVO();
        ProductInformationDO productInformationDO = productInformationMapper.selectById(idVO.getId());
        ProductInformationVO productInformationVO = new ProductInformationVO();
        BeanUtils.copyProperties(productInformationDO,productInformationVO);
        productDataVO.setProductInformationVO(productInformationVO);
        ProductContractDO productContractDO = productContractMapper.selectByProductId(idVO.getId());
        if(productContractDO!=null){
            ProductContractVO productContractVO = new ProductContractVO();
            BeanUtils.copyProperties(productContractDO,productContractVO);
            productDataVO.setProductContractVO(productContractVO);
        }
        ProductDerivativeDO productDerivativeDO = productDerivativeMapper.selectByProductId(idVO.getId());
        if(productDerivativeDO != null){
            ProductDerivativeVO productDerivativeVO = new ProductDerivativeVO();
            BeanUtils.copyProperties(productDerivativeDO,productDerivativeVO);
            productDataVO.setProductDerivativeVO(productDerivativeVO);
        }
        List<InvestmentManagerDO> investmentManagerDOList = investmentManagerMapper.selectByRegisterNumber(productInformationDO.getRegisterNumber());
        if(!investmentManagerDOList.isEmpty()){
            List<InvestmentManagerVO> investmentManagerVOS = new ArrayList<>();
            for (InvestmentManagerDO investmentManagerDO : investmentManagerDOList) {
                InvestmentManagerVO investmentManagerVO = new InvestmentManagerVO();
                BeanUtils.copyProperties(investmentManagerDO,investmentManagerVO);
                investmentManagerVOS.add(investmentManagerVO);
            }
            productDataVO.setInvestmentManagerVOList(investmentManagerVOS);
        }
        return new ResultVo(ResultCode.SUCCESS,productDataVO);
    }

    @Override
    public List<ProductDataVO> selectProuctInformationByIdList(List<Integer> idList) {
        List<ProductDataVO> result = new ArrayList<>();
        List<ProductInformationDO> productInformationDOList = new ArrayList<>();
        if(idList.isEmpty()){
            productInformationDOList = productInformationMapper.selectAllRecord();
        }else{
            productInformationDOList = productInformationMapper.selectByIdList(idList);
        }
        for (ProductInformationDO productInformationDO : productInformationDOList) {
            ProductDataVO productDataVO = new ProductDataVO();
            ProductInformationVO productInformationVO = new ProductInformationVO();
            BeanUtils.copyProperties(productInformationDO,productInformationVO);
            productDataVO.setProductInformationVO(productInformationVO);
            ProductContractDO productContractDO = productContractMapper.selectByProductId(productInformationDO.getId());
            if(productContractDO!=null){
                ProductContractVO productContractVO = new ProductContractVO();
                BeanUtils.copyProperties(productContractDO,productContractVO);
                productDataVO.setProductContractVO(productContractVO);
            }
            ProductDerivativeDO productDerivativeDO = productDerivativeMapper.selectByProductId(productInformationDO.getId());
            if(productDerivativeDO != null){
                ProductDerivativeVO productDerivativeVO = new ProductDerivativeVO();
                BeanUtils.copyProperties(productDerivativeDO,productDerivativeVO);
                productDataVO.setProductDerivativeVO(productDerivativeVO);
            }
            List<InvestmentManagerDO> investmentManagerDOList = investmentManagerMapper.selectByRegisterNumber(productInformationDO.getRegisterNumber());
            if(!investmentManagerDOList.isEmpty()){
                List<InvestmentManagerVO> investmentManagerVOS = new ArrayList<>();
                for (InvestmentManagerDO investmentManagerDO : investmentManagerDOList) {
                    InvestmentManagerVO investmentManagerVO = new InvestmentManagerVO();
                    BeanUtils.copyProperties(investmentManagerDO,investmentManagerVO);
                    investmentManagerVOS.add(investmentManagerVO);
                }
                productDataVO.setInvestmentManagerVOList(investmentManagerVOS);
            }
            result.add(productDataVO);
        }
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo removeProduct(List<Integer> idList) {
        ResultVo vo = new ResultVo(ResultCode.DELETE_SUCCESS);
        vo.setData(true);
        vo.setMsg(ResultCode.DELETE_SUCCESS.getMsg());
        Integer userId = UserUtils.getLoginUser().getUserId();
        List<ProductInformationDO>  productInformationDOList = productInformationMapper.selectByIdList(idList);
        for (ProductInformationDO productInformationDO : productInformationDOList) {
            productInformationMapper.deleteByProductId(productInformationDO.getId(),userId);
            productContractMapper.deleteByProductId(productInformationDO.getId(),userId);
            productDerivativeMapper.deleteByProductId(productInformationDO.getId(),userId);
            investmentManagerMapper.deleteByRegisterNumber(List.of(productInformationDO.getRegisterNumber()));
        }
        return vo;
    }

    private ProductUploadResult parseResult(ProductData productData) {
        ProductUploadResult result = new ProductUploadResult();
        int productStartRow = 8;
        List<ProductExcelSuccessDataVO> successDataVOList = new ArrayList<>();
        List<ProductExcelFailDataVO> excelFailDataVOList = new ArrayList<>();
        //产品基本信息
        List<ProductInformationData> productInformationDataList = productData.getProductInformationDataList();
        saveProductInformation(productInformationDataList,successDataVOList,excelFailDataVOList,productStartRow);
        //合同
        List<ProductContractData> productContractDataList = productData.getProductContractDataList();
        saveProductContractData(productContractDataList,successDataVOList,excelFailDataVOList,productStartRow);
        //其他信息
        List<ProductDerivativeData> productDerivativeDOList = productData.getProductDerivativeDataList();
        saveProductDerivativeData(productDerivativeDOList,successDataVOList,excelFailDataVOList,productStartRow);
        //投资经理信息
        List<InvestmentManagerData> investmentManagerDataList = productData.getInvestmentManagerDataList();
        productStartRow = 3;
        saveInvestmentManagerData(investmentManagerDataList,successDataVOList,excelFailDataVOList,productStartRow);
        result.setSuccess(successDataVOList);
        result.setFail(excelFailDataVOList);
        result.setTotal(successDataVOList.size()+excelFailDataVOList.size());
        result.setFailCount(excelFailDataVOList.size());
        result.setSuccessCount(successDataVOList.size());
        return result;
    }

    private void saveInvestmentManagerData(List<InvestmentManagerData> investmentManagerDataList,
                                           List<ProductExcelSuccessDataVO> successDataVOList,
                                           List<ProductExcelFailDataVO> excelFailDataVOList,
                                           int productStartRow) {
        List<InvestmentManagerDO> investmentManagerDOList = new ArrayList<>();
        for (int idx =0;idx< investmentManagerDataList.size();idx++) {
            InvestmentManagerData investmentManagerData = investmentManagerDataList.get(idx);
            if(StringUtil.isEmpty(investmentManagerData.getRegisterNumber()) && StringUtil.isEmpty(investmentManagerData.getManagerName())){
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.INVESTMENT_MANAGER_SHEET_NAME,ExcelConst.DELETION_MANAGER_REGISTERENUMBER,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            InvestmentManagerDO investmentManagerDO = new InvestmentManagerDO();
            investmentManagerDO.setRegisterNumber(investmentManagerData.getRegisterNumber());
            investmentManagerDO.setStartDate(investmentManagerData.getStartDate());
            investmentManagerDO.setEndDate(investmentManagerData.getEndDate());
            investmentManagerDO.setManagerName(investmentManagerData.getManagerName());
            investmentManagerDO.setUpdatetime(new Date());
            investmentManagerDO.setUpdaterid(UserUtils.getLoginUser().getUserId());
            investmentManagerDO.setIsvalid(1);
            investmentManagerDO.setCreatetime(new Date());
            investmentManagerDO.setCreatorid(UserUtils.getLoginUser().getUserId());
            investmentManagerDOList.add(investmentManagerDO);
            ProductExcelSuccessDataVO successDataVO = new ProductExcelSuccessDataVO();
            successDataVO.setRowNum(idx+productStartRow);
            successDataVO.setSheet(ExcelConst.INVESTMENT_MANAGER_SHEET_NAME);
            successDataVO.setParseStatus(ExcelConst.SUCCESS);
            successDataVOList.add(successDataVO);
        }
        List<String> registerNumberList = investmentManagerDOList.stream().map(InvestmentManagerDO::getRegisterNumber).collect(Collectors.toList());
        if(!registerNumberList.isEmpty()){
            investmentManagerMapper.deleteByRegisterNumber(registerNumberList);
        }
        try{
            investmentManagerMapper.insert(investmentManagerDOList);
        }catch (Exception e){
            logger.error(e.getMessage());
        }
    }

    private void saveProductDerivativeData(List<ProductDerivativeData> productDerivativeDOList,
                                           List<ProductExcelSuccessDataVO> successDataVOList,
                                           List<ProductExcelFailDataVO> excelFailDataVOList,
                                           int productStartRow) {
        for (int idx =0;idx< productDerivativeDOList.size();idx++) {
            ProductDerivativeData productDerivativeData = productDerivativeDOList.get(idx);
            if(StringUtil.isEmpty(productDerivativeData.getRegisterNumber()) && StringUtil.isEmpty(productDerivativeData.getProductName())){
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_CONTRACT_SHEET_NAME,ExcelConst.DELETION_NAME_REGISTERENUMBER,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            ProductDerivativeDO productDerivativeDO = toProductDerivativeDO(productDerivativeData);
            ProductInformationDO productInformationDO = null;
            if(!StringUtil.isEmpty(productDerivativeDO.getProductName())){
                productInformationDO = productInformationMapper.selectByRegisterNumber(productDerivativeDO.getRegisterNumber());
            }else{
                productInformationDO = productInformationMapper.selectByProductName(productDerivativeDO.getProductName());
            }
            if(productInformationDO != null){
                productDerivativeDO.setProductId(productInformationDO.getId());
            }else{
                productDerivativeDO.setCreatetime(new Date());
                productDerivativeDO.setCreatorid(UserUtils.getLoginUser().getUserId());
            }
            ProductDerivativeDO oldProductDerivativeDO = productDerivativeMapper.selectByNameAndRegisterNumber(productDerivativeData.getRegisterNumber(),productDerivativeData.getProductName());
            if(oldProductDerivativeDO != null){
                productDerivativeDO.setId(oldProductDerivativeDO.getId());
            }
            try{
                productDerivativeMapper.insertOrUpdate(productDerivativeDO);
                ProductExcelSuccessDataVO successDataVO = new ProductExcelSuccessDataVO();
                successDataVO.setRowNum(idx+productStartRow);
                successDataVO.setSheet(ExcelConst.PRODUCT_DETAIL_SHEET_NAME);
                successDataVO.setParseStatus(ExcelConst.SUCCESS);
                successDataVOList.add(successDataVO);
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_CONTRACT_SHEET_NAME,ExcelConst.SAVE_FAIL,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
            }
        }
    }

    private ProductDerivativeDO toProductDerivativeDO(ProductDerivativeData productDerivativeData) {
        ProductDerivativeDO productDerivativeDO = new ProductDerivativeDO();
        productDerivativeDO.setProductName(productDerivativeData.getProductName());
        productDerivativeDO.setRegisterNumber(productDerivativeData.getRegisterNumber());
        productDerivativeDO.setDistributeReport(productDerivativeData.getDistributeReport());
        productDerivativeDO.setCumulativeNav(productDerivativeData.getCumulativeNav());
        productDerivativeDO.setInvestmentManagerDesc(productDerivativeData.getInvestmentManagerDesc());
        productDerivativeDO.setFeeNote(productDerivativeData.getFeeNote());
        productDerivativeDO.setManageAsset(productDerivativeData.getManageAsset());
        productDerivativeDO.setManagementfeeBank(productDerivativeData.getManagementfeeBank());
        productDerivativeDO.setManagementfeeTrust(productDerivativeData.getManagementfeeTrust());
        productDerivativeDO.setOutsourceFee(productDerivativeData.getOutsourceFee());
        productDerivativeDO.setProductCount(productDerivativeData.getProductCount());
        productDerivativeDO.setProductAsset(productDerivativeData.getProductAsset());
        productDerivativeDO.setProductShare(productDerivativeData.getProductShare());
        productDerivativeDO.setProductValuation(productDerivativeData.getProductValuation());
        productDerivativeDO.setStartDate(productDerivativeData.getStartDate());
        return productDerivativeDO;
    }

    private void saveProductContractData(List<ProductContractData> productContractDataList, 
                                         List<ProductExcelSuccessDataVO> successDataVOList, 
                                         List<ProductExcelFailDataVO> excelFailDataVOList, 
                                         int productStartRow) {
        for (int idx =0;idx< productContractDataList.size();idx++) {
            ProductContractData productContractData = productContractDataList.get(idx);
            if(StringUtil.isEmpty(productContractData.getRegisterNumber()) && StringUtil.isEmpty(productContractData.getProductName())){
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_CONTRACT_SHEET_NAME,ExcelConst.DELETION_NAME_REGISTERENUMBER,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            ProductContractDO productContractDO = toProductContractDO(productContractData);
            ProductInformationDO productInformationDO = null;
            if(!StringUtil.isEmpty(productContractDO.getProductName())){
                productInformationDO = productInformationMapper.selectByRegisterNumber(productContractDO.getRegisterNumber());
            }else{
                productInformationDO = productInformationMapper.selectByProductName(productContractDO.getProductName());
            }
            if(productInformationDO != null){
                productContractDO.setProductId(productInformationDO.getId());
            }else{
                productContractDO.setCreatetime(new Date());
                productContractDO.setCreatorid(UserUtils.getLoginUser().getUserId());
            }
            ProductContractDO oldProductContractDO = productContractMapper.selectByNameAndRegisterNumber(productContractData.getRegisterNumber(),productContractData.getProductName());
            if(oldProductContractDO != null){
                productContractDO.setId(oldProductContractDO.getId());
            }
            try{
                productContractMapper.insertOrUpdate(productContractDO);
                ProductExcelSuccessDataVO successDataVO = new ProductExcelSuccessDataVO();
                successDataVO.setRowNum(idx+productStartRow);
                successDataVO.setSheet(ExcelConst.PRODUCT_CONTRACT_SHEET_NAME);
                successDataVO.setParseStatus(ExcelConst.SUCCESS);
                successDataVOList.add(successDataVO);
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_CONTRACT_SHEET_NAME,ExcelConst.SAVE_FAIL,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
            }
        }
    }

    private ProductContractDO toProductContractDO(ProductContractData productContractData) {
        ProductContractDO productContractDO = new ProductContractDO();
        productContractDO.setRegisterNumber(productContractData.getRegisterNumber());
        productContractDO.setProductName(productContractData.getProductName());
        productContractDO.setAccruedMethod(productContractData.getAccruedMethod());
        productContractDO.setInvestmentLimit(productContractData.getInvestmentLimit());
        productContractDO.setInvestmentMethod(productContractData.getInvestmentMethod());
        productContractDO.setInvestmentScope(productContractData.getInvestmentScope());
        productContractDO.setInvestmentStrategy(productContractData.getInvestmentStrategy());
        productContractDO.setPerformanceBasic(productContractData.getPerformanceBasic());
        productContractDO.setUpdatetime(new Date());
        productContractDO.setProductContract(productContractData.getProductContract());
        productContractDO.setRemark(productContractData.getRemark());
        productContractDO.setUpdaterid(UserUtils.getLoginUser().getUserId());
        productContractDO.setIsvalid(1);
        return productContractDO;
    }

    private void saveProductInformation(List<ProductInformationData> productInformationDataList, List<ProductExcelSuccessDataVO> successDataVOList, List<ProductExcelFailDataVO> excelFailDataVOList, int productStartRow) {
        for (int idx =0;idx< productInformationDataList.size();idx++) {
            ProductInformationData productInformationData = productInformationDataList.get(idx);
            if(StringUtil.isEmpty(productInformationData.getId())){
                continue;
            }
            if(!isNumber(productInformationData.getId())){
                continue;
            }
            if(StringUtil.isEmpty(productInformationData.getRegisterNumber()) && StringUtil.isEmpty(productInformationData.getProductName())){
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_SHEET_NAME,ExcelConst.DELETION_NAME_REGISTERENUMBER,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            try{
                Date date = DateUtils.parse(productInformationData.getInceptionDate(),DateUtils.YYYY_MM_DD);
                if(date.after(new Date())){
                    ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_SHEET_NAME,ExcelConst.ERROR_INCEPTION_DATE,idx+productStartRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
            }catch (Exception e){
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_SHEET_NAME,ExcelConst.INCEPTION_ERROR_DATE_FORMAT,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            ProductInformationDO productInformationDO = toProductDO(productInformationData);
            ProductInformationDO oldProductInformationDO = null;
            if(!StringUtil.isEmpty(productInformationDO.getProductName())){
                oldProductInformationDO = productInformationMapper.selectByRegisterNumber(productInformationDO.getRegisterNumber());
            }else{
                oldProductInformationDO = productInformationMapper.selectByProductName(productInformationDO.getProductName());
            }
            if(oldProductInformationDO != null){
                productInformationDO.setId(oldProductInformationDO.getId());
            }else{
                productInformationDO.setCreatetime(new Date());
                productInformationDO.setCreatorid(UserUtils.getLoginUser().getUserId());
            }
            try{
                productInformationMapper.insertOrUpdate(productInformationDO);
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                ProductExcelFailDataVO failDataVO = toExcelFailDataVO(ExcelConst.PRODUCT_SHEET_NAME,ExcelConst.SAVE_FAIL,idx+productStartRow);
                excelFailDataVOList.add(failDataVO);
                continue;
            }
            ProductExcelSuccessDataVO successDataVO = new ProductExcelSuccessDataVO();
            successDataVO.setRowNum(idx+productStartRow);
            successDataVO.setSheet(ExcelConst.PRODUCT_SHEET_NAME);
            successDataVO.setParseStatus(ExcelConst.SUCCESS);
            successDataVOList.add(successDataVO);
        }
    }

    private ProductInformationDO toProductDO(ProductInformationData productInformationData) {
        ProductInformationDO productInformationDO = new ProductInformationDO();
        productInformationDO.setProductName(productInformationData.getProductName());
        productInformationDO.setRegisterNumber(productInformationData.getRegisterNumber());
        productInformationDO.setAssetFrequency(productInformationData.getAssetFrequency());
        productInformationDO.setNavFrequency(productInformationData.getNavFrequency());
        productInformationDO.setFirstStrategy(productInformationData.getFirstStrategy());
        productInformationDO.setSecondStrategy(productInformationData.getSecondStrategy());
        productInformationDO.setThirdStrategy(productInformationData.getThirdStrategy());
        productInformationDO.setInceptionDate(productInformationData.getInceptionDate());
        productInformationDO.setInvestmentManager(productInformationData.getInvestmentManager());
        productInformationDO.setInvestmentStrategyDesc(productInformationData.getInvestmentStrategyDesc());
        productInformationDO.setReportFrequency(productInformationData.getReportFrequency());
        productInformationDO.setRemark(productInformationData.getRemark());
        productInformationDO.setMsLabel(productInformationData.getMsLabel());
        productInformationDO.setStartDate(productInformationData.getStartDate());
        productInformationDO.setLiquidateDate(productInformationData.getLiquidateDate());
        productInformationDO.setIsStruct(productInformationData.getIsStruct());
        productInformationDO.setUpdatetime(new Date());
        productInformationDO.setUpdaterid(UserUtils.getLoginUser().getUserId());
        productInformationDO.setIsvalid(1);
        return productInformationDO;
    }

    public static boolean isNumber(String str) {
        if (StringUtils.isEmpty(str)) {
            return false;
        }
        if ("%".equals(str)) {
            return true;
        }
        str = str.replace(",", "").replace("，", "");
        //容许没有整数部分 eg:.2
        String regex = "^-?(\\d+)?(\\.)?\\d+%?$";

        boolean judge = str.matches(regex);
        // 非常规数值，可能为科学计数法
        if (!judge) {
            try {
                String plainString = new BigDecimal(str).toPlainString();
            } catch (Exception e) {
                return false;
            }
        }
        return true;
    }

    private ProductExcelFailDataVO toExcelFailDataVO(String sheetName, String msg,Integer rowNum) {
        ProductExcelFailDataVO failDataVO = new ProductExcelFailDataVO();
        failDataVO.setFailReason(msg);
        failDataVO.setRowNum(rowNum);
        failDataVO.setSheet(sheetName);
        return failDataVO;
    }

    private ProductData parseProductFile(File file) {
        // 创建一个 list 存储每行的数据，即 ExcelData 对象
        List<ProductInformationData> productInformationDataList = new ArrayList<>();
        // 直接使用 EasyExcel 的 read 方法，同时定义表头的类型，以便将列中数据映射为 ExcelData 对象
        EasyExcel.read(file, ProductInformationData.class, new PageReadListener<ProductInformationData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (ProductInformationData excelData : dataList) {
                if (excelData != null) {
                    productInformationDataList.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet(0).headRowNumber(3).doRead();

        List<ProductContractData> productContractDataList = new ArrayList<>();
        EasyExcel.read(file, ProductContractData.class, new PageReadListener<ProductContractData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (ProductContractData excelData : dataList) {
                if (excelData != null) {
                    productContractDataList.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet(1).headRowNumber(3).doRead();

        List<ProductDerivativeData> productDerivativeDataList = new ArrayList<>();
        EasyExcel.read(file, ProductDerivativeData.class, new PageReadListener<ProductDerivativeData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (ProductDerivativeData excelData : dataList) {
                if (excelData != null) {
                    productDerivativeDataList.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet(2).headRowNumber(3).doRead();

        List<InvestmentManagerData> investmentManagerDataList = new ArrayList<>();
        EasyExcel.read(file, InvestmentManagerData.class, new PageReadListener<InvestmentManagerData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (InvestmentManagerData excelData : dataList) {
                if (excelData != null) {
                    investmentManagerDataList.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet(3).headRowNumber(3).doRead();
        ProductData productData = new ProductData();
        productData.setProductInformationDataList(productInformationDataList);
        productData.setProductContractDataList(productContractDataList);
        productData.setProductDerivativeDataList(productDerivativeDataList);
        productData.setInvestmentManagerDataList(investmentManagerDataList);
        return productData;
    }
}
